      subroutine manipu ( CGNS_file_index )
c
c     routine to manipulate a cgns base
c
      implicit none
c
      include 'cgnslib_f.h'
c
c arguments
      integer      CGNS_file_index
c
c local
      integer      nbases
      integer      base_in,base_out
      integer      imanip
      integer      cell_dim,phys_dim
      integer      ier
      integer      nzones,nzone
      integer      ncoords,ncoord,ncoord_out,maxvar
      integer      datatype,datatype_save
      integer      zone_out,zonetype,location
      integer      size(6),size_out(6)
      integer      size1_in,size2_in,size3_in
      integer      size1_out,size2_out,size3_out
      integer      nfield, nfields, nfield_MomY, nfield_MomZ, nfield_out
      integer      range_min(3), range_max(3)
      integer      n,i,j,k
      integer      rot_num
      integer      imass, ilength, itime, itemp, iangle
      integer      dataclass
      integer      nsol, nsols, nsol_out
      integer      len, narray, narrays, DataDim, DimVec
      real*8       rot_deg, theta,dtheta
      real*8       PI, deg2rad
      integer      DataArrayI
      real         DataArrayR
      real         cscale,coffset
      double precision DataArrayD
      integer,     allocatable:: DataArrayI(:)
      character*80 base_name
      character*80 base_name_in
      character*32 zone_name, solname, arrayname
c
      real,             allocatable:: var_R(:,:,:)
      double precision, allocatable:: var_D(:,:,:)
      double precision, allocatable:: var_in(:,:,:,:)
      double precision, allocatable:: var_out(:,:,:,:)
      character*32,     allocatable:: var_name(:)
      character*1,      allocatable:: StateDesc(:)

c     define some constants
      PI = dacos(-1.d0)
      deg2rad = PI/180.d0
c
c     Choose base to manipulate
      call cgns_list_bases ( CGNS_file_index, nbases )
      if ( nbases .le. 0 ) then
        write(6,*) '  There are no bases in the working file to'
     &            ,' manipulate'
        write(6,*) '  Read a base first'
        go to 500
      else if ( nbases .eq. 1 ) then
        write(6,*) '  Only one base found... Selecting base 1'
        base_in = 1
      else
        write(6,*) '  Enter the base you would like to manipulate'
        write(6,*) '    (creating a new base)'
        read(5,*) base_in
      end if
c
c     enter a name for the new base
      write(6,*) '  Enter a name for the base being created'
      read(5,*) base_name
c
c     select manipulation requested
   50 continue
      write(6,*)
      write(6,*) '  Available manipulations:'
      write(6,*) ' 0 - return'
      write(6,*) ' 1 - copy base'
      write(6,*) ' 2 - 2D base rotated about x axis to get 3D base'
      write(6,*) '  Enter manipulation requested'
      read(5,*) imanip
      if ( imanip .eq. 0 ) then
c       oops, should not be here I guess, return
        return
      else if ( imanip .eq. 1 ) then
        write(6,*) '  actually, I do not know how to do this, try again'
        go to 50
      else if ( imanip .eq. 2 ) then
c
c       get other info needed
        write(6,*) '  Enter number of circumferential points'
        read(5,*) rot_num
        write(6,*) '  Enter total degrees of rotation (e.g. 360 or 180)'
        read(5,*) rot_deg
c       Convert to radians
        rot_deg = rot_deg*deg2rad
c
      else
        write(6,*) '  this option does not exist... Try again'
        go to 50
      end if
c
c     check that base_in has the right properties
      call cg_base_read_f ( CGNS_file_index, base_in, base_name_in,
     &                      cell_dim, phys_dim, ier )
      if ( ier .ne. 0 ) call cg_error_exit_f
      if ( imanip .eq. 2 ) then
c
c       we are going to rotate a 2D grid about the x axis
c       however, assume the grid was written '3d' with kdim=1
c           but, phys_dim can be 2 or 3
c       cell_dim = number of grid indices (2 for 2D, 3 for 3D)
c       phys_dim = number of physical dimensions ( 2 for XY, 3 for XYZ) 
        if ( cell_dim .ne. 3 ) then
          write(6,*) '  manipu: error: cell_dim=2 not allowed for now'
          go to 500
        end if
c
      end if
c
c     create the new base node
c     assume adding a 3D base for now
      cell_dim = 3
      phys_dim = 3
      write(6,*) '  Assuming creating a 3D base'
      call cg_base_write_f ( CGNS_file_index, base_name, cell_dim,
     &                       phys_dim, base_out, ier )
      if ( ier .ne. 0 ) call cg_error_exit_f
c
c     Write out dataclass information 
c     ( how non-dimensionalized) and the dimensional units
c
c     First go to the input base solution
      call cg_goto_f( CGNS_file_index, base_in, ier,'end')
      if ( ier .ne. 0 ) call cg_error_exit_f
c
c       Now get the units
        call cg_units_read_f( imass, ilength, itime, itemp, 
     &                              iangle, ier)
        if ( ier .eq. 0 ) then
        call cg_goto_f( CGNS_file_index, base_out, ier, 'end')
        if ( ier .ne. 0 ) call cg_error_exit_f
c
c       Now write this information to the new base
        call cg_units_write_f( imass, ilength, itime, itemp, 
     &                                 iangle, ier)
        if ( ier .ne. 0 ) call cg_error_exit_f
      endif
c
c     Now get the dataclass
      call cg_goto_f( CGNS_file_index, base_in, ier, 'end')
      if ( ier .ne. 0 ) call cg_error_exit_f
c
      call cg_dataclass_read_f( dataclass, ier)
      if ( ier .eq. 0 ) then
        call cg_goto_f( CGNS_file_index, base_out, ier, 'end')
        if ( ier .ne. 0 ) call cg_error_exit_f
c
c       Now write this information to the new base
        call cg_dataclass_write_f( dataclass, ier)
        if ( ier .ne. 0 ) call cg_error_exit_f
      endif
c
c     determine number of zones in the base
      call cg_nzones_f ( CGNS_file_index, base_in, nzones, ier )
      if ( ier .ne. 0 ) call cg_error_exit_f
      write(6,*) '  There are ',nzones,' zones in the base'
      write(6,*) '  Assuming performing same operation on all zones'
c
c     Get reference state description from input base ...
      call cg_goto_f(CGNS_file_index, base_in, ier, 'end')
      if ( ier .ne. 0 ) call cg_error_exit_f
c     Read and copy ReferenceState
      call cg_state_size_f(len, ier)
      if ( ier .eq. 1 .or. ier .eq. 3 ) then
        call cg_error_exit_f
      else if ( ier .eq. 2 ) then
        write(6,*) '  manipu: warning: No StateDescription found',
     &             '  assuming this is OK.'
      else if ( ier .eq. 0 ) then
        allocate ( StateDesc(len) )
        call cg_state_read_f(StateDesc, ier)
        if ( ier .ne. 0 ) call cg_error_exit_f
c       ... and now write to output base
        call cg_goto_f(CGNS_file_index, base_out, ier,'end')
        if ( ier .ne. 0 ) call cg_error_exit_f
        call cg_state_write_f(StateDesc, ier)
        deallocate(StateDesc)
c
c       Write out dataclass information ( how non-dimensionalized)
c       and the dimensional units
c
c       First go to the input base grid coordinates
        call cg_goto_f( CGNS_file_index, base_in, ier, 
     &                 'ReferenceState_t', 1, 'end')
        if ( ier .ne. 0 ) call cg_error_exit_f
c
c       Now get the units
        call cg_units_read_f( imass, ilength, itime, itemp, 
     &                              iangle, ier)
        if ( ier .eq. 0 ) then
          call cg_goto_f( CGNS_file_index, base_out, ier, 
     &                 'ReferenceState_t', 1, 'end')
          if ( ier .ne. 0 ) call cg_error_exit_f
c
c         Now write this information to the new base
          call cg_units_write_f( imass, ilength, itime, itemp, 
     &                           iangle, ier)
          if ( ier .ne. 0 ) call cg_error_exit_f
        endif
c
c       Now get the dataclass
        call cg_goto_f( CGNS_file_index, base_in, ier, 
     &                 'ReferenceState_t', 1, 'end')
        if ( ier .ne. 0 ) call cg_error_exit_f
c
        call cg_dataclass_read_f( dataclass, ier)
        if ( ier .eq. 0 ) then
          call cg_goto_f( CGNS_file_index, base_out, ier, 
     &                 'ReferenceState_t', 1, 'end')
          if ( ier .ne. 0 ) call cg_error_exit_f
c
c         Now write this information to the new base
          call cg_dataclass_write_f( dataclass, ier)
          if ( ier .ne. 0 ) call cg_error_exit_f
        endif
c
c       Goto the input base and find out how many data arrays
c       there are.  We will loop over the narrays, reading from
c       the input base, going to and writing to the output base.
        call cg_goto_f(CGNS_file_index, base_in, ier,
     &                 'ReferenceState_t', 1, 'end')
        if ( ier .ne. 0 ) call cg_error_exit_f
        call cg_narrays_f(narrays, ier)
        if ( ier .ne. 0 ) call cg_error_exit_f
c       This little hack in place so that the darn code runs!!
c       opening unit 99, but not doing anything with it. Don't ask!!
        open(99,file='/dev/null')
        do narray = 1, narrays
          call cg_goto_f(CGNS_file_index, base_in, ier,
     &              'ReferenceState_t', 1, 'end')
          call cg_array_info_f(narray, arrayname, datatype,
     &                         DataDim, DimVec, ier)
          if ( ier .ne. 0) call cg_error_exit_f
          if (DataDim .gt. 1 .or. DimVec .gt. 1) then
            write(6,*)'  Manipu: error: assuming reference state',
     &                '  data is scalar (not vector)' 
            go to 500
          endif

c         Read and write the DataArray
          if (datatype .eq. Integer) then
            call cg_array_read_f(narray, DataArrayI, ier)
            if ( ier .ne. 0 ) call cg_error_exit_f

            call cg_goto_f(CGNS_file_index, base_out, ier, 
     &                           'ReferenceState_t', 1, 'end')
            if ( ier .ne. 0) call cg_error_exit_f
            call cg_array_write_f(arrayname, datatype,
     &                          DataDim, DimVec, DataArrayI, ier)
            if ( ier .ne. 0 ) call cg_error_exit_f
          elseif (datatype .eq. RealSingle) then
            call cg_array_read_f(narray, DataArrayR, ier)
            if ( ier .ne. 0 ) call cg_error_exit_f

            call cg_goto_f(CGNS_file_index, base_out, ier,
     &                           'ReferenceState_t', 1, 'end')
            if ( ier .ne. 0) call cg_error_exit_f
            call cg_array_write_f(arrayname, datatype,
     &                                DataDim, DimVec, DataArrayR, ier)
            if ( ier .ne. 0 ) call cg_error_exit_f
          elseif (datatype .eq. RealDouble) then
            call cg_array_read_f(narray, DataArrayD, ier)
            if ( ier .ne. 0 ) call cg_error_exit_f

            call cg_goto_f(CGNS_file_index, base_out, ier, 
     &                           'ReferenceState_t', 1, 'end')
            if ( ier .ne. 0) call cg_error_exit_f
            call cg_array_write_f(arrayname, datatype,
     &                                DataDim, DimVec, DataArrayD, ier)
            if ( ier .ne. 0 ) call cg_error_exit_f
          endif
        enddo
        close(99)
      else
        write(6,*) '  manipu: warning: No StateDescription found',
     &             '  assuming this is OK.'
      endif
c
c     loop over zones
      do nzone = 1,nzones
c
c       get size of the array
        call cg_zone_read_f ( CGNS_file_index, base_in, nzone,
     &                        zone_name, size, ier )
        if ( ier .ne. 0 ) call cg_error_exit_f
        call cg_zone_type_f ( CGNS_file_index, base_in, nzone,
     &                        zonetype, ier)
        if ( ier .ne. 0 ) call cg_error_exit_f
        range_min(1) = 1
        range_min(2) = 1
        range_min(3) = 1
        range_max(1) = size(1)
        range_max(2) = size(2)
        range_max(3) = size(3)
c
        if ( imanip .eq. 2 ) then
c
c         define zone in new base
          size_out(1) = size(1)
          size_out(2) = size(2)
          size_out(3) = rot_num
          size_out(4) = size(4)
          size_out(5) = size(5)
          size_out(6) = rot_num-1 
          call cg_zone_write_f(CGNS_file_index, base_out, zone_name,
     &                         size_out, zonetype, zone_out, ier )
          if ( ier .ne. 0 ) call cg_error_exit_f
          if ( zone_out .ne. nzone ) then
            write(6,*) '  manipu: error: somehow zone numbers do not,
     &                   match'
            go to 500
          end if
c
c         determine number of coordinate arrays in the base
          call cg_ncoords_f ( CGNS_file_index, base_in, nzone,
     &                        ncoords, ier )
          if ( ier .ne. 0 ) call cg_error_exit_f
c
          if ( ncoords .gt. 0 ) then
c
            maxvar = ncoords
            allocate ( var_in ( size(1),size(2),size(3), maxvar ) )
            allocate ( var_out( size(1),size(2),size_out(3), maxvar ) )
            allocate ( var_name( maxvar) )
c
c           read coordinates
c           ncoord is either X, Y, or Z
            do 100 ncoord = 1,ncoords
              call cg_coord_info_f ( CGNS_file_index, base_in, nzone,
     &                               ncoord, datatype, var_name(ncoord),
     &                               ier )
              if ( ier .ne. 0 ) call cg_error_exit_f
              if ( ncoord .eq. 1 ) then
                datatype_save = datatype
              else
                if ( datatype .ne. datatype_save ) then
                  write(6,*) '  manipu: error: currently assume'
     &                      ,' datatype is the same for all coords'
                  write(6,*) '    (want to manip and write as a group)'
                  go to 500
                endif
              end if
              if ( ncoord .eq. 1 ) then
                if ( trim(var_name(ncoord)) .ne. 'CoordinateX' ) then
                  write(6,*) '  manipu: error: currently assume XYZ'
                  go to 500
                endif
              elseif ( ncoord .eq. 2 ) then
                if ( trim(var_name(ncoord)) .ne. 'CoordinateY' ) then
                  write(6,*) '  manipu: error: currently assume XYZ'
                  go to 500
                endif
              elseif ( ncoord .eq. 3 ) then
                if ( trim(var_name(ncoord)) .ne. 'CoordinateZ' ) then
                  write(6,*) '  manipu: error: currently assume XYZ'
                  go to 500
                endif
              endif
c
              if ( datatype .eq. RealSingle ) then
                allocate ( var_R ( size(1),size(2),size(3) ) )
                call cg_coord_read_f ( CGNS_file_index, base_in, nzone,
     &                                 var_name(ncoord), datatype, 
     &                                 range_min, range_max, var_R, ier)
                if ( ier .ne. 0 ) call cg_error_exit_f
                do k=1,size(3)
                do j=1,size(2)
                do i=1,size(1)
                  var_in(i,j,k,ncoord) = var_R(i,j,k)
                end do
                end do
                end do
                deallocate ( var_R )
              else if ( datatype .eq. RealDouble ) then
                allocate ( var_D ( size(1),size(2),size(3) ) )
                call cg_coord_read_f ( CGNS_file_index, base_in, nzone,
     &                                 var_name(ncoord), datatype, 
     &                                 range_min, range_max, var_D, ier)
                if ( ier .ne. 0 ) call cg_error_exit_f
                do k=1,size(3)
                do j=1,size(2)
                do i=1,size(1)
                  var_in(i,j,k,ncoord) = var_D(i,j,k)
                end do
                end do
                end do
                deallocate ( var_D )
              else
                write(6,*) '  manipu: error: bad datatype for coords'
                write(6,*) ' base,zone,coord ',base_in,nzone,ncoord
                go to 500
              end if
 100        continue
c
c           manipulate coordinates
            do ncoord=1,ncoords
              do k=1,size(3)
              do j=1,size(2)
              do i=1,size(1)
                var_out(i,j,k,ncoord) = var_in(i,j,k,ncoord)
              end do
              end do
              end do
            end do
c
c           over-write coord 3 based on rotation about x
can         Ashish fix this
            dtheta = rot_deg/dfloat(size_out(3)-1)
            do k=1,size_out(3)
              theta = dfloat(k-1)*dtheta
              do j=1,size(2)
              do i=1,size(1)
                var_out(i,j,k,1) = var_in(i,j,1,1)
                var_out(i,j,k,2) = var_in(i,j,1,2)*dcos(theta)
                var_out(i,j,k,3) = var_in(i,j,1,2)*dsin(theta)
              end do
              end do
            end do
c
c           write coordinates to new base
            do 200 ncoord = 1,ncoords
              if ( datatype .eq. RealSingle ) then
                allocate ( var_R (size_out(1),size_out(2),size_out(3)))
                do k=1,size_out(3)
                do j=1,size_out(2)
                do i=1,size_out(1)
                  var_R(i,j,k) = var_out(i,j,k,ncoord)
                end do
                end do
                end do
                call cg_coord_write_f ( CGNS_file_index, base_out,
     &                               nzone, datatype, var_name(ncoord),
     &                               var_R, ncoord_out, ier)
                if ( ier .ne. 0 ) call cg_error_exit_f
                if ( ncoord_out .ne. ncoord ) then
                  write(6,*)'  manipu: error: the coordinate number'
                  write(6,*)'  written by CGNS is not correct'
                  go to 500
                endif
                deallocate ( var_R )
              else if ( datatype .eq. RealDouble ) then
                allocate ( var_D (size_out(1),size_out(2),size_out(3)))
                do k=1,size_out(3)
                do j=1,size_out(2)
                do i=1,size_out(1)
                  var_D(i,j,k) = var_out(i,j,k,ncoord)
                end do
                end do
                end do
                call cg_coord_write_f ( CGNS_file_index, base_out,
     &                               nzone, datatype, var_name(ncoord),
     &                               var_D, ncoord_out, ier)
                if ( ier .ne. 0 ) call cg_error_exit_f
                if ( ncoord_out .ne. ncoord ) then
                  write(6,*)'  manipu: error: the coordinate number'
                  write(6,*)'  written by CGNS is not correct'
                  go to 500
                endif
                deallocate ( var_D )
              else
                write(6,*) '  manipu: error: bad datatype for coords'
                write(6,*) ' base,zone,coord ',base_in,nzone,ncoord
                go to 500
              end if

c
c             Write out dataclass information ( how non-dimensionalized)
c             and the dimensional units
c
c             First go to the input base grid coordinates
c             WARNING:  no way to get number of GridCoordinates sets
c             For now ASSUMING it is 1
              call cg_goto_f( CGNS_file_index, base_in, ier, 
     &                    'Zone_t', nzone, 'GridCoordinates_t', 1,
     &                    'DataArray_t', ncoord, 'end') 
              if ( ier .ne. 0 ) call cg_error_exit_f
c
c             Now get the units
              call cg_units_read_f( imass, ilength, itime, itemp, 
     &                              iangle, ier)
              if ( ier .eq. 0 ) then
                call cg_goto_f( CGNS_file_index, base_out, ier, 
     &                    'Zone_t', nzone, 'GridCoordinates_t', 1,
     &                    'DataArray_t', ncoord, 'end') 
                if ( ier .ne. 0 ) call cg_error_exit_f
c
c             Now write this information to the new base
                call cg_units_write_f( imass, ilength, itime, itemp, 
     &                                 iangle, ier)
                if ( ier .ne. 0 ) call cg_error_exit_f
              endif
c
c             Now get the dataclass
              call cg_goto_f( CGNS_file_index, base_in, ier, 
     &                    'Zone_t', nzone, 'GridCoordinates_t', 1,
     &                    'DataArray_t', ncoord, 'end') 
              if ( ier .ne. 0 ) call cg_error_exit_f
c
              call cg_dataclass_read_f( dataclass, ier)
              if ( ier .eq. 0 ) then
                call cg_goto_f( CGNS_file_index, base_out, ier, 
     &                    'Zone_t', nzone, 'GridCoordinates_t', 1,
     &                    'DataArray_t', ncoord, 'end') 
                if ( ier .ne. 0 ) call cg_error_exit_f
c
c             Now write this information to the new base
                call cg_dataclass_write_f( dataclass, ier)
                if ( ier .ne. 0 ) call cg_error_exit_f
              endif
c
c             Now get any data conversions
              call cg_goto_f( CGNS_file_index, base_in, ier, 
     &                    'Zone_t', nzone, 'GridCoordinates_t', 1,
     &                    'DataArray_t', ncoord, 'end') 
              if ( ier .ne. 0 ) call cg_error_exit_f
c
              call cg_conversion_info_f ( datatype, ier )
              if ( ier .eq. 0 ) then
c               assume conversion data exists
                if ( datatype .ne. RealSingle ) then
                  write(6,*) '  manipu.f: error: assume datatype'
                  write(6,*) '    for conversion info is Real'
                  go to 500
                end if
                call cg_conversion_read_f ( cscale,coffset,ier )
                if ( ier .ne. 0 ) call cg_error_exit_f
c
                call cg_goto_f( CGNS_file_index, base_out, ier, 
     &                    'Zone_t', nzone, 'GridCoordinates_t', 1,
     &                    'DataArray_t', ncoord, 'end') 
                if ( ier .ne. 0 ) call cg_error_exit_f
c
c               Now write this information to the new base
                call cg_conversion_write_f( datatype, 
     &               cscale,coffset, ier)
                if ( ier .ne. 0 ) call cg_error_exit_f
              end if

 200        continue          ! loops over ncoord
c
            deallocate (var_in)
            deallocate (var_out)
            deallocate (var_name)
          end if
c
c         do the same for solutions
c
c         determine number of solution variables in the base
          call cg_nsols_f ( CGNS_file_index, base_in, nzone,
     &                        nsols, ier )
          if ( ier .ne. 0 ) call cg_error_exit_f
c
          if ( nsols .gt. 0 ) then
	    write(*,*)'Solutions found. Proceeding with manipulation'
c           read solution variables
            write(*,*)'reading the solution'
            do 300 nsol = 1,nsols
c             Get the name of solution and how it is
c             stored (either CellCenter or Vertex)
              call cg_sol_info_f ( CGNS_file_index, base_in, nzone,
     &                               nsol, solname, location,
     &                               ier )
              if ( ier .ne. 0 ) call cg_error_exit_f
c
c             Create a base for the new solution
              call cg_sol_write_f ( CGNS_file_index, base_out, nzone,
     &                              solname, location, nsol_out, ier)
              if ( ier .ne. 0 ) call cg_error_exit_f
              if ( nsol_out .ne. nsol ) then
                write(6,*)'  manipu: error:  solution output is not',
     &                    '  equal to the input solution number'
                go to 500
              endif

              if ( location .eq. CellCenter ) then
                range_max(1) = range_max(1)-1
                range_max(2) = range_max(2)-1
                range_max(3) = range_max(3)-1
                size1_in = size(4)
                size2_in = size(5)
                size3_in = size(6)
                size1_out = size_out(4)
                size2_out = size_out(5)
                size3_out = size_out(6)
              else if ( location .eq. Vertex ) then
                size1_in = size(1)
                size2_in = size(2)
                size3_in = size(3)
                size1_out = size_out(1)
                size2_out = size_out(2)
                size3_out = size_out(3)
              endif
c             Determine how many variables (or fields) there are in
c             each solution
              call cg_nfields_f(CGNS_file_index, base_in, nzone, nsol,
     &                          nfields, ier)
              if ( ier .ne. 0 ) call cg_error_exit_f
c
              allocate (var_in(size1_in,size2_in,size3_in, nfields))
              allocate (var_out(size1_out,size2_out,size3_out, nfields))
              allocate (var_name(nfields))
 
              nfield_MomY = 0
              nfield_MomZ = 0
              do 301 nfield = 1, nfields
c               Get the name of each solution variable
                call cg_field_info_f(CGNS_file_index, base_in, nzone,
     &                               nsol, nfield, datatype,
     &                               var_name(nfield), ier)
                if ( ier .ne. 0 ) call cg_error_exit_f

c               Get the nfield number that corresponds to the 
c               Y momentum or v velocity
                if (var_name(nfield) .eq. 'MomentumY' .or.
     &              var_name(nfield) .eq. 'VelocityY') nfield_MomY = 1

c               Get the nfield number that corresponds to the 
c               Z momentum or w velocity
                if (var_name(nfield) .eq. 'MomentumZ' .or.
     &              var_name(nfield) .eq. 'VelocityZ') nfield_MomZ = 1
                  
                if ( nfield .eq. 1 ) then
                  datatype_save = datatype
                else
                  if ( datatype .ne. datatype_save ) then
                    write(6,*) '  manipu: error: currently assume'
     &                        ,' datatype is the same for all fields'
                    write(6,*) '  (want to manip and write as a group)'
                    go to 500
                  endif
                end if
c
                if ( datatype .eq. RealSingle ) then
                  allocate ( var_R ( size1_in,size2_in,size3_in ) )
                  call cg_field_read_f ( CGNS_file_index, base_in, 
     &                                 nzone, nsol,
     &                                 var_name(nfield), datatype, 
     &                                 range_min, range_max, var_R, ier)
                  if ( ier .ne. 0 ) call cg_error_exit_f
                  do k=1,size3_in
                  do j=1,size2_in
                  do i=1,size1_in
                    var_in(i,j,k,nfield) = var_R(i,j,k)
                  end do
                  end do
                  end do
                  deallocate ( var_R )
                else if ( datatype .eq. RealDouble ) then
                  allocate ( var_D ( size1_in,size2_in,size3_in ) )
                  call cg_field_read_f ( CGNS_file_index, base_in, 
     &                                 nzone, nsol,
     &                                 var_name(nfield), datatype, 
     &                                 range_min, range_max, var_D, ier)
                  if ( ier .ne. 0 ) call cg_error_exit_f
                  do k=1,size3_in
                  do j=1,size2_in
                  do i=1,size1_in
                    var_in(i,j,k,nfield) = var_D(i,j,k)
                  end do
                  end do
                  end do
                  deallocate ( var_D )
                else
                  write(6,*) '  manipu: error: bad datatype for field'
                  write(6,*) ' base,zone,field ',base_in,nzone,nfield
                  go to 500
                end if
 301          continue
                  
              if (nfield_MomY .eq. 0) then
                write(6,*)'  manipu: error: Neither Y Momentum nor',
     &                    '  Y velocity found.  Cannot rotate'
                go to 500
              end if

              if (nfield_MomZ .eq. 0) then
                write(6,*)'  manipu: error: Neither Z Momentum nor',
     &                    '  Z velocity found.  create new field',
     &                    '  assuming original base has Z component'
                go to 500
              end if
c
c             manipulate variables
              write(*,*)'rotating the variables'
              dtheta = rot_deg/dfloat(size_out(3)-1)
              do k=1,size3_out
                theta = dfloat(k-1)*dtheta
                do j=1,size2_out
                  do i=1,size1_out
                    do nfield=1,nfields
                      if (var_name(nfield) .eq. 'MomentumY' .or.
     &                    var_name(nfield) .eq. 'VelocityY') then
                        var_out(i,j,k,nfield) = 
     &                             var_in(i,j,1,nfield_MomY)*dsin(theta)
                      else if (var_name(nfield) .eq. 'MomentumZ') then
                        var_out(i,j,k,nfield) = 
     &                             var_in(i,j,1,nfield_MomY)*dcos(theta)
                      else
                        var_out(i,j,k,nfield) = var_in(i,j,1,nfield)
                      endif
                    end do
                  end do
                end do
              end do
c
c             write solution to new base
              write(*,*)'writing the variables'
              do 302 nfield = 1,nfields
                if ( datatype .eq. RealSingle ) then
                  allocate (var_R(size1_out,size2_out,size3_out))
                  do k=1,size3_out
                  do j=1,size2_out
                  do i=1,size1_out
                    var_R(i,j,k) = var_out(i,j,k,nfield)
                  end do
                  end do
                  end do
                  call cg_field_write_f ( CGNS_file_index, base_out,
     &                               nzone, nsol, datatype,
     &                               var_name(nfield),
     &                               var_R, nfield_out, ier)
                  if ( ier .ne. 0 ) call cg_error_exit_f
                  if ( nfield_out .ne. nfield ) then
                    write(6,*)'  manipu: error: the field number'
                    write(6,*)'  written by CGNS is not correct'
                    go to 500
                  endif
                  deallocate ( var_R )
                else if ( datatype .eq. RealDouble ) then
                  allocate ( var_D (size1_out,size2_out,size3_out))
                  do k=1,size3_out
                  do j=1,size2_out
                  do i=1,size1_out
                    var_D(i,j,k) = var_out(i,j,k,nfield)
                  end do
                  end do
                  end do
                  call cg_field_write_f ( CGNS_file_index, base_out,
     &                               nzone, nsol, datatype, 
     &                               var_name(nfield),
     &                               var_D, nfield_out, ier)
                  if ( ier .ne. 0 ) call cg_error_exit_f
                  if ( nfield_out .ne. nfield ) then
                    write(6,*)'  manipu: error: the field number'
                    write(6,*)'  written by CGNS is not correct'
                    go to 500
                  endif
                  deallocate ( var_D )
                else
                  write(6,*) '  manipu: error: bad datatype for fields'
                  write(6,*) ' base,zone,field ',base_in,nzone,nfield
                  go to 500
                end if
c
c               Write out dataclass information 
c               ( how non-dimensionalized) and the dimensional units
c
c               First go to the input base solution
                call cg_goto_f( CGNS_file_index, base_in, ier, 
     &                      'Zone_t', nzone, 'FlowSolution_t', nsol,
     &                      'DataArray_t', nfield, 'end')
                if ( ier .ne. 0 ) call cg_error_exit_f
c
c               Now get the units
                call cg_units_read_f( imass, ilength, itime, itemp, 
     &                              iangle, ier)
                if ( ier .eq. 0 ) then
                  call cg_goto_f( CGNS_file_index, base_out, ier, 
     &                      'Zone_t', nzone, 'FlowSolution_t', nsol,
     &                      'DataArray_t', nfield, 'end')
                  if ( ier .ne. 0 ) call cg_error_exit_f
c
c               Now write this information to the new base
                  call cg_units_write_f( imass, ilength, itime, itemp, 
     &                                 iangle, ier)
                  if ( ier .ne. 0 ) call cg_error_exit_f
                endif
c
c               Now get the dataclass
                call cg_goto_f( CGNS_file_index, base_in, ier, 
     &                      'Zone_t', nzone, 'FlowSolution_t', nsol,
     &                      'DataArray_t', nfield, 'end') 
                if ( ier .ne. 0 ) call cg_error_exit_f
c
                call cg_dataclass_read_f( dataclass, ier)
                if ( ier .eq. 0 ) then
                  call cg_goto_f( CGNS_file_index, base_out, ier, 
     &                      'Zone_t', nzone, 'FlowSolution_t', 1,
     &                      'DataArray_t', nfield, 'end') 
                  if ( ier .ne. 0 ) call cg_error_exit_f
c
c                 Now write this information to the new base
                  call cg_dataclass_write_f( dataclass, ier)
                  if ( ier .ne. 0 ) call cg_error_exit_f
                endif
c
c               Now get any data conversions
                call cg_goto_f( CGNS_file_index, base_in, ier, 
     &                      'Zone_t', nzone, 'FlowSolution_t', nsol,
     &                      'DataArray_t', nfield, 'end') 
                if ( ier .ne. 0 ) call cg_error_exit_f
c
                call cg_conversion_info_f ( datatype, ier )
                if ( ier .eq. 0 ) then
c                 assume conversion data exists
                  if ( datatype .ne. RealSingle ) then
                    write(6,*) '  manipu.f: error: assume datatype'
                    write(6,*) '    for conversion info is Real'
                    go to 500
                  end if
                  call cg_conversion_read_f ( cscale,coffset,ier )
                  if ( ier .ne. 0 ) call cg_error_exit_f
c
                  call cg_goto_f( CGNS_file_index, base_out, ier, 
     &                      'Zone_t', nzone, 'FlowSolution_t', 1,
     &                      'DataArray_t', nfield, 'end') 
                  if ( ier .ne. 0 ) call cg_error_exit_f
c
c                 Now write this information to the new base
                  call cg_conversion_write_f( datatype, 
     &                 cscale,coffset, ier)
                  if ( ier .ne. 0 ) call cg_error_exit_f
                end if

 302          continue          ! loops over nfields
c
              deallocate (var_in)
              deallocate (var_out)
              deallocate (var_name)
 300        continue            ! loops over nsols
          end if
c         ignore bc information for now
c
c         error out if section data
c
c
        end if
      end do
c
  500 continue

      return
      end
